from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.workspace_role import WorkspaceRole
from ..types import UNSET, Unset

T = TypeVar("T", bound="AddWorkspaceUsersRequest")


@_attrs_define
class AddWorkspaceUsersRequest:
    """Request to add users to a workspace (POST /workspace/{id}/users).

    All invited users receive the same role.

        Attributes:
            emails (list[str]):
            role (WorkspaceRole | Unset): Role of a user within a workspace.
    """

    emails: list[str]
    role: WorkspaceRole | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        emails = self.emails

        role: str | Unset = UNSET
        if not isinstance(self.role, Unset):
            role = self.role.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "emails": emails,
            }
        )
        if role is not UNSET:
            field_dict["role"] = role

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        emails = cast(list[str], d.pop("emails"))

        _role = d.pop("role", UNSET)
        role: WorkspaceRole | Unset
        if isinstance(_role, Unset):
            role = UNSET
        else:
            role = WorkspaceRole(_role)

        add_workspace_users_request = cls(
            emails=emails,
            role=role,
        )

        add_workspace_users_request.additional_properties = d
        return add_workspace_users_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
