from __future__ import annotations

from collections.abc import Mapping
from typing import Any, Literal, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="StreamStartEvent")


@_attrs_define
class StreamStartEvent:
    """Provides the assistant message ID so frontend can set up the response container.

    Attributes:
        assistant_message_ext_id (str):
        type_ (Literal['stream_start'] | Unset):  Default: 'stream_start'.
    """

    assistant_message_ext_id: str
    type_: Literal["stream_start"] | Unset = "stream_start"
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        assistant_message_ext_id = self.assistant_message_ext_id

        type_ = self.type_

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "assistant_message_ext_id": assistant_message_ext_id,
            }
        )
        if type_ is not UNSET:
            field_dict["type"] = type_

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        assistant_message_ext_id = d.pop("assistant_message_ext_id")

        type_ = cast(Literal["stream_start"] | Unset, d.pop("type", UNSET))
        if type_ != "stream_start" and not isinstance(type_, Unset):
            raise ValueError(f"type must match const 'stream_start', got '{type_}'")

        stream_start_event = cls(
            assistant_message_ext_id=assistant_message_ext_id,
            type_=type_,
        )

        stream_start_event.additional_properties = d
        return stream_start_event

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
