from http import HTTPStatus
from typing import Any

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.http_validation_error import HTTPValidationError
from ...models.upload_email_body import UploadEmailBody
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    body: UploadEmailBody | Unset = UNSET,
    user_email: str,
    subject: str | Unset = "",
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    params: dict[str, Any] = {}

    params["user_email"] = user_email

    params["subject"] = subject

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/api/document/upload-email",
        "params": params,
    }

    if not isinstance(body, Unset):
        _kwargs["files"] = body.to_multipart()

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Any | HTTPValidationError | None:
    if response.status_code == 200:
        response_200 = response.json()
        return response_200

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[Any | HTTPValidationError]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient | Client,
    body: UploadEmailBody | Unset = UNSET,
    user_email: str,
    subject: str | Unset = "",
) -> Response[Any | HTTPValidationError]:
    """Upload Email

     Receive email from the gateway and process it.

    Authentication: Bearer JWT from CI service account, or legacy X-API-Key.

    Two paths:
    - **No document attachments (Path A)**: Calls LLM with email text, sends reply
      email, creates notification. No workspace created.
    - **With document attachments (Path B)**: Creates temp workspace, processes
      documents through the full pipeline, then (async) calls LLM with parsed
      document content, sends reply email, stores conversation, and creates
      notification when all documents finish processing.

    The gateway sends `email_body.txt` as a file containing the email text.
    All other files are treated as document attachments.

    Args:
        user_email (str):
        subject (str | Unset):  Default: ''.
        body (UploadEmailBody | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any | HTTPValidationError]
    """

    kwargs = _get_kwargs(
        body=body,
        user_email=user_email,
        subject=subject,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient | Client,
    body: UploadEmailBody | Unset = UNSET,
    user_email: str,
    subject: str | Unset = "",
) -> Any | HTTPValidationError | None:
    """Upload Email

     Receive email from the gateway and process it.

    Authentication: Bearer JWT from CI service account, or legacy X-API-Key.

    Two paths:
    - **No document attachments (Path A)**: Calls LLM with email text, sends reply
      email, creates notification. No workspace created.
    - **With document attachments (Path B)**: Creates temp workspace, processes
      documents through the full pipeline, then (async) calls LLM with parsed
      document content, sends reply email, stores conversation, and creates
      notification when all documents finish processing.

    The gateway sends `email_body.txt` as a file containing the email text.
    All other files are treated as document attachments.

    Args:
        user_email (str):
        subject (str | Unset):  Default: ''.
        body (UploadEmailBody | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Any | HTTPValidationError
    """

    return sync_detailed(
        client=client,
        body=body,
        user_email=user_email,
        subject=subject,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient | Client,
    body: UploadEmailBody | Unset = UNSET,
    user_email: str,
    subject: str | Unset = "",
) -> Response[Any | HTTPValidationError]:
    """Upload Email

     Receive email from the gateway and process it.

    Authentication: Bearer JWT from CI service account, or legacy X-API-Key.

    Two paths:
    - **No document attachments (Path A)**: Calls LLM with email text, sends reply
      email, creates notification. No workspace created.
    - **With document attachments (Path B)**: Creates temp workspace, processes
      documents through the full pipeline, then (async) calls LLM with parsed
      document content, sends reply email, stores conversation, and creates
      notification when all documents finish processing.

    The gateway sends `email_body.txt` as a file containing the email text.
    All other files are treated as document attachments.

    Args:
        user_email (str):
        subject (str | Unset):  Default: ''.
        body (UploadEmailBody | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any | HTTPValidationError]
    """

    kwargs = _get_kwargs(
        body=body,
        user_email=user_email,
        subject=subject,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient | Client,
    body: UploadEmailBody | Unset = UNSET,
    user_email: str,
    subject: str | Unset = "",
) -> Any | HTTPValidationError | None:
    """Upload Email

     Receive email from the gateway and process it.

    Authentication: Bearer JWT from CI service account, or legacy X-API-Key.

    Two paths:
    - **No document attachments (Path A)**: Calls LLM with email text, sends reply
      email, creates notification. No workspace created.
    - **With document attachments (Path B)**: Creates temp workspace, processes
      documents through the full pipeline, then (async) calls LLM with parsed
      document content, sends reply email, stores conversation, and creates
      notification when all documents finish processing.

    The gateway sends `email_body.txt` as a file containing the email text.
    All other files are treated as document attachments.

    Args:
        user_email (str):
        subject (str | Unset):  Default: ''.
        body (UploadEmailBody | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Any | HTTPValidationError
    """

    return (
        await asyncio_detailed(
            client=client,
            body=body,
            user_email=user_email,
            subject=subject,
        )
    ).parsed
